/*
 * noddyencd.c - Encode a txt file into Noddy format.
 *
 * Copyright (C) 2015 Paul Daniels <paul.daniels40@yahoo.co.uk>
 *
 * Takes a txt file and encodes multiple Noddy pages. Outputs a binary
 * file that can be included into a mtx file.
 *
 * Format of input file:
 *
 * `
 * Page name
 * <23 lines by 39 chars>
 * 
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * 11/09/2015 - PD Created
 *
 */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>

void	help_msg ();
void	encode_noddy (FILE *, FILE *);

main	(int argc, char **argv)

{

FILE	*infile, *outfile;
int	prmcnt;
unsigned char	*outname, *outfilename, *outfileext;

if (argc < 2) {
  help_msg ();
  return (0);
}

prmcnt = 1;

if (!(infile = fopen (argv[prmcnt], "rb"))) {
  printf ("encodenod: Could not open file - '%s'.\n", argv[prmcnt]);
  return (-1);
}

/* Make copy of input filename */

outname = malloc (strlen (argv[prmcnt]) + 5);
strncpy (outname, argv[prmcnt], strlen(argv[prmcnt]) + 5);

/* Find file name component of input filename */

outfilename = strrchr (outname, '/');
if (outfilename == NULL) {
	outfilename = outname;
}

/* Find extention of input filename */

outfileext = strrchr (outfilename, '.');

if (outfileext == NULL) {
	strcat (outfilename, ".bin");
} else {
	strcpy (outfileext, ".bin");
}

printf ("Output filename is %s.\n", outname);

if (!(outfile = fopen (outname, "wb+"))) {
  printf ("encodenod: Could not open file - '%s'.\n", outname);
  return (-1);
}

encode_noddy (infile, outfile);

fclose (infile);
fclose (outfile);
free (outname);

}

/******************************************************************************
 * Usage / Help message
 ******************************************************************************/

void	help_msg ()

{

printf ("Usage: encodenod <file>\n");

printf ("\nBy Paul Daniels. Version 0.1\n");

}

/******************************************************************************
 * Encode the pages in the input file
 *
 * Input Page Format:
 *
 * `
 * PAGE NAME
 * Page content
 * `
 * PAGE NAME
 * Page content
 * <eof>
 *
 * Noddy Page Format:
 *
 * Page Length		16 bit. Includes length, name, contents and terminator.
 * Page Name		Last char has bit 7 high
 * Page contents	Spaces replaced by 80 + number of spaces.
 * Page terminator	0xff
 * 
 * Note a page is 39 columns x 24 lines.
 ******************************************************************************/

void	encode_noddy (FILE *infile, FILE *outfile)

{

int		page_len, name_flg, lines, runlen, tmplen;
int		loop;
unsigned char	buffer[43];
unsigned char	page[1024];

memset (page, 0, 1024);
page_len = 2;
name_flg = 0;
lines = runlen = 0;

while (!feof (infile)) {

  if (fgets (buffer, 42, infile) != NULL) {

    if (buffer[0] == '`') {

      /* End previous page if one has been started */

      runlen = 0;

      if (page_len != 2) {
	page[page_len++] = 0xff;
	page[0] = page_len & 0xff;
	page[1] = (page_len >> 8) & 0xff;
	fwrite (page, page_len, 1, outfile);
	page_len = 2;
	name_flg = 0;
	lines = 0;
      }

    } else if (!name_flg) {

      buffer[strcspn(buffer, " \r\n")] = 0;
      tmplen = strlen (buffer);

      /* Output the page name */

      printf ("Noddy Page: %s.\n", buffer);

      for (loop = 0; loop < tmplen; loop++) {
	page[page_len++] = buffer[loop];
      }
      page[page_len - 1] = page[page_len - 1] | 0x80;
      name_flg = 1;

    } else {

      buffer[strcspn(buffer, "\r\n")] = 0;
      tmplen = strlen (buffer);

      /* Pad line with spaces */

      for (loop = tmplen; loop < 42; loop++) buffer[loop] = ' ';

      /* Need to stop after reading 24 lines */
      
      if (lines > 23) {
	printf ("Too many lines, ignored.\n");

      } else {

	/* Output the page content */

	for (loop = 0; loop < 39; loop++) {

	  if (buffer[loop] && buffer[loop] != ' ') {

	    while (runlen) {
	      if (runlen > 126) {
		page[page_len++] = 0xfe;
	      	runlen -= 126;
	      } else {
		page[page_len++] = 0x80 | runlen;
		runlen = 0;
	      }
	    }
	    page[page_len++] = buffer[loop];

	  } else {

	    /* Need to runlength compress spaces over multiple lines */

	    runlen++;

	  }
	}

	lines++;
      }
    }
  }
}

/* End previous page if on has been started */

if (page_len != 2) {
  page[page_len++] = 0xff;
  page[0] = page_len & 0xff;
  page[1] = (page_len >> 8) & 0xff;
  fwrite (page, page_len, 1, outfile);
}

}
